local WorldQuestGroupFinderAddon = CreateFrame("Frame", "WorldQuestGroupFinderAddon", UIParent)
local _, L = ...;

WorldQuestGroupFinder = {}
WorldQuestGroupFinderConfig = {}

local DefaultConfig = {}
local RegisteredEvents = {}
WorldQuestGroupFinderAddon:SetScript("OnEvent", function (self, event, ...) if (RegisteredEvents[event]) then return RegisteredEvents[event](self, event, ...) end end)

local name = "WorldQuestGroupCurrentWQFrame"
local currentWQFrame = CreateFrame("frame",name,UIParent)
	
local currentWQ = nil
local tempWQ = nil
local playerFullName = UnitName("player").."-"..GetRealmName()

local BROADCAST_PREFIX = "WQGF"

local activityIDs = {
	["419"] = "Aszuna",
	["420"] = "Val'sharah", 
	["421"] = "Highmountain",
	["422"] = "Stormheim", 
	["423"] = "Suramar"
}

local pendingApplications = {}
local blacklistedLeaders = {}
local seenWorldQuests = {}

function RegisteredEvents:ADDON_LOADED(event, addon, ...)
	if (addon == "WorldQuestGroupFinder") then
		RegisterAddonMessagePrefix(BROADCAST_PREFIX)
		SLASH_WQGF1 = '/wqgf'
		SlashCmdList["WQGF"] = function (msg, editbox)
			WorldQuestGroupFinder.handleCMD(msg, editbox)	
		end
		setmetatable(WorldQuestGroupFinderConfig, {__index = DefaultConfig})
	end
end

function RegisteredEvents:PLAYER_LOGIN(event)
	print("|c00bfffffWorld Quest Group Finder v0.5 BETA. "..L["Click on a world quest in the objective tracking window to search for a group."])
	WorldQuestGroupFinder.SetHooks()
	WorldQuestGroupFinderConf.CreateConfigMenu()
end

function RegisteredEvents:PLAYER_ENTERING_WORLD(event)
	if (WorldQuestGroupFinderConf.GetConfigValue("savedCurrentWQ") ~= nil and currentWQ == nil) then
		C_Timer.After(1, function()
			if (IsInGroup() and C_LFGList.GetActiveEntryInfo()) then
				WorldQuestGroupFinder.HandleWorldQuestStart(WorldQuestGroupFinderConf.GetConfigValue("savedCurrentWQ"))
			else
				WorldQuestGroupFinderConf.SetConfigValue("savedCurrentWQ", nil)
			end
		end)
	end
end

function RegisteredEvents:LFG_LIST_APPLICANT_LIST_UPDATED(event, hasNewPending, hasNewPendingWithData)
	if ( currentWQ ~= nil and WorldQuestGroupFinderConf.GetConfigValue("autoinviteUsers")) then
		if ( hasNewPending and hasNewPendingWithData and LFGListUtil_IsEntryEmpowered() ) then
			local isLeader = UnitIsGroupLeader("player");
			local autoAccept = select(9, C_LFGList.GetActiveEntryInfo());
			if (isLeader) then 
				local applicants = C_LFGList.GetApplicants();
				for i=1, #applicants do
					local id, status, pendingStatus, numMembers, isNew, comment = C_LFGList.GetApplicantInfo(applicants[i]);
					if (status == "applied" and (GetNumGroupMembers() + numMembers) <= 5 and comment == "WorldQuestGroupFinder User") then
						C_LFGList.InviteApplicant(id)
						if (numMembers > 1) then
							WorldQuestGroupFinder.prefixedPrint(L["A World Quest Group Finder users group has automatically been accepted!"])
						else
							WorldQuestGroupFinder.prefixedPrint(L["A World Quest Group Finder user has automatically been accepted!"])
						end
					end			
				end
			end
		end
	end
end

function RegisteredEvents:CHAT_MSG_ADDON(event, prefix, message, channel, sender)
	if (prefix == BROADCAST_PREFIX and sender ~= playerFullName) then
		if (string.find(message, "#WQS:")) then
			local tempWQ = string.gsub(message, "#WQS:(.*)#", "%1")
			if(IsQuestFlaggedCompleted(tempWQ)) then
				WorldQuestGroupFinder.prefixedPrint(string.format(L["Your group is now doing the world quest |c00bfffff%s|c00ffffff. You already have completed this world quest."], WorldQuestGroupFinder.GetQuestInfo(tempWQ)))
			else
				if (WorldQuestGroupFinder.HandleWorldQuestStart(tempWQ)) then
					WorldQuestGroupFinder.prefixedPrint(string.format(L["Your group is now doing the world quest |c00bfffff%s|c00ffffff."], WorldQuestGroupFinder.GetQuestInfo(tempWQ)))
				else 
					WorldQuestGroupFinder.prefixedPrint(string.format(L["Your group is now doing the world quest |c00bfffff%s|c00ffffff. You are not eligible to do this world quest."], WorldQuestGroupFinder.GetQuestInfo(tempWQ)))
				end
			end	
		elseif (string.find(message, "#WQE:")) then
			local tempWQ = string.gsub(message, "#WQE:(.*)#", "%1")
			WorldQuestGroupFinder.prefixedPrint(string.format(L["Your group is no longer doing the world quest |c00bfffff%s|c00ffffff."], WorldQuestGroupFinder.GetQuestInfo(tempWQ)))
			if (currentWQ == tempWQ) then
				WorldQuestGroupFinder.HandleWorldQuestEnd(tempWQ)
			end
		end
	end
end

function RegisteredEvents:QUEST_TURNED_IN(event, questID, experience, money)
	if (currentWQ ~= nil and questID == currentWQ) then
		if (WorldQuestGroupFinderConf.GetConfigValue("notifyParty") and IsInGroup()) then
			WorldQuestGroupFinder.SendWQCompletionPartyNotification(questID)
		end
		if (WorldQuestGroupFinderConf.GetConfigValue("askToLeave")) then
			WorldQuestGroupFinder.ShowLeavePrompt()
		end
		WorldQuestGroupFinder.HandleWorldQuestEnd(currentWQ)
	end
end

function RegisteredEvents:LFG_LIST_APPLICATION_STATUS_UPDATED(event, applicationID, status)
	local _,_,name,description,_,ilvl,_,_,_,_,_,_,author,members = C_LFGList.GetSearchResultInfo(applicationID)
	if (status == "applied") then
		pendingApplications[applicationID] = tempWQ
	end
	if (status == "declined" or status == "failed" or status == "invitedeclined" or status == "timedout") then
		blacklistedLeaders[author] = true
		-- If all applications have been declined, restart process. A new group will be created if nothing else is found.
		if (C_LFGList.GetNumApplications() - 1 == 0) then
			WorldQuestGroupFinder.InitWQGroup(pendingApplications[applicationID])
		end
	end
	if (status == "cancelled") then
		-- placeholder
	end
	if (status == "inviteaccepted") then
		WorldQuestGroupFinder.prefixedPrint(string.format(L["You have joined |c00bfffff%s|c00ffffff's group for |c00bfffff%s|c00ffffff. Have fun!"], author, WorldQuestGroupFinder.GetQuestInfo(pendingApplications[applicationID])))
		WorldQuestGroupFinder.HandleWorldQuestStart(pendingApplications[applicationID])
	end
end

function RegisteredEvents:GROUP_ROSTER_UPDATE(event)
	-- Leaving the group. TRIGGERED TWICE
	if (IsInGroup() == false and currentWQ ~= nil) then
		WorldQuestGroupFinder.HandleWorldQuestEnd(currentWQ)
	end
end

for k, v in pairs(RegisteredEvents) do
	WorldQuestGroupFinderAddon:RegisterEvent(k)
end

function WorldQuestGroupFinder.GetQuestInfo (questID)
	local tagID, tagName, worldQuestType, rarity, elite, tradeskillLineIndex = GetQuestTagInfo (questID)
	local title, factionID = C_TaskQuest.GetQuestInfoByQuestID (questID)
	return title, factionID, tagID, tagName, worldQuestType, rarity, elite, tradeskillLineIndex
end

function WorldQuestGroupFinder.SetHooks()	
	local bkp_BonusObjectiveTracker_OnBlockClick = BonusObjectiveTracker_OnBlockClick
	function BonusObjectiveTracker_OnBlockClick(self, button)	
		if (button == "LeftButton") then
			if (IsInGroup() and not UnitIsGroupLeader("player")) then
				WorldQuestGroupFinder.prefixedPrint(L["You are not the group leader."])
				return false
			end
			tempWQ = self.TrackedQuest.questID
			if (currentWQ ~= nil and tempWQ ~= currentWQ) then
				StaticPopup_Show ("NEW_WORLD_QUEST_PROMPT")
			else 
				-- No current WQ. 
				if (currentWQ == nil) then
					WorldQuestGroupFinder.InitWQGroup(tempWQ)
				else
					WorldQuestGroupFinder.prefixedPrint(L["You already are in a group for this world quest."])
				end
			end
		end
		bkp_BonusObjectiveTracker_OnBlockClick(self, button)
	end
	
	-- BonusObjectiveTracker_OnBlockEnter ?
	local bkp_BonusObjectiveTracker_OnBlockAnimInFinished = BonusObjectiveTracker_OnBlockAnimInFinished
	function BonusObjectiveTracker_OnBlockAnimInFinished(self) 
		if (WorldQuestGroupFinderConf.GetConfigValue("askZoning")) then
			local block = self:GetParent();
			local title, factionID, tagID, tagName, worldQuestType, rarity, elite, tradeskillLineIndex = WorldQuestGroupFinder.GetQuestInfo(block.id)
			-- Ignore pet battle and dungeon quests
			if (worldQuestType ~= LE_QUEST_TAG_TYPE_PET_BATTLE and worldQuestType ~= LE_QUEST_TAG_TYPE_DUNGEON) then
				tempWQ = block.id
				if (not IsInGroup() and WorldQuestGroupFinderConf.GetConfigValue("askZoningAuto")) then
					WorldQuestGroupFinder.InitWQGroup(tempWQ)
				else
					-- Check if the world quest zone has already been entered during this session
					if not seenWorldQuests[block.id] then
						-- No current WQ. 
						if (currentWQ == nil) then
							seenWorldQuests[block.id] = true
							StaticPopup_Show ("WORLD_QUEST_ENTERED_PROMPT", title)
						-- Already doing another WQ
						elseif (currentWQ ~= nil and tempWQ ~= currentWQ) then
							if not WorldQuestGroupFinderConf.GetConfigValue("askZoningBusy") then
								seenWorldQuests[block.id] = true
								StaticPopup_Show ("WORLD_QUEST_ENTERED_SWITCH_PROMPT", title)
							end
						end
					end
				end
			end
		end
		bkp_BonusObjectiveTracker_OnBlockAnimInFinished(self) 
	end

	local bkp_BonusObjectiveTracker_TrackWorldQuest = BonusObjectiveTracker_TrackWorldQuest
	function BonusObjectiveTracker_TrackWorldQuest(questID, hardWatch)
		bkp_BonusObjectiveTracker_TrackWorldQuest(questID, hardWatch)
	end
	
	local bkp_C_LFGList_RemoveListing = C_LFGList.RemoveListing
	function C_LFGList.RemoveListing()
		if (currentWQ ~= nil) then
			WorldQuestGroupFinder.HandleWorldQuestEnd(currentWQ)
		end
		bkp_C_LFGList_RemoveListing()
	end
	
	local bkp_BonusObjectiveTracker_OnOpenDropDown = BonusObjectiveTracker_OnOpenDropDown
	function BonusObjectiveTracker_OnOpenDropDown(self)		
		local block = self.activeFrame;
		local questID = block.TrackedQuest.questID;
		
		if (tonumber(questID) ~= tonumber(currentWQ)) then
			local searchGroup = UIDropDownMenu_CreateInfo();
			searchGroup.notCheckable = true;

			searchGroup.text = L["Search or Create Group"];
			searchGroup.func = function()
				WorldQuestGroupFinder.InitWQGroup(questID);
			end;

			searchGroup.checked = false;
			UIDropDownMenu_AddButton(searchGroup, UIDROPDOWN_MENU_LEVEL);
			bkp_BonusObjectiveTracker_OnOpenDropDown(self)
		else 
			local searchGroup = UIDropDownMenu_CreateInfo();
			searchGroup.notCheckable = true;

			searchGroup.text = L["Stop grouping for this world quest"];
			searchGroup.func = function()
				if (IsInGroup() and not UnitIsGroupLeader("player")) then
					WorldQuestGroupFinder.prefixedPrint(L["You are not the group leader."])
				else
					C_LFGList.RemoveListing()
				end
			end;

			searchGroup.checked = false;
			UIDropDownMenu_AddButton(searchGroup, UIDROPDOWN_MENU_LEVEL);
		end
	end
end

function WorldQuestGroupFinder.AddWorldQuestToTracker(questID)
	AddWorldQuestWatch(questID, true) 
end

function WorldQuestGroupFinder.InitWQGroup(wqID) 
	local title, factionID, tagID, tagName, worldQuestType, rarity, elite, tradeskillLineIndex = WorldQuestGroupFinder.GetQuestInfo(wqID)
	local foundZone = false
	local foundGroup = false
	local currentZone = nil
	if (IsInGroup() and not UnitIsGroupLeader("player")) then
		WorldQuestGroupFinder.prefixedPrint(L["You are not the group leader."])
		return false
	end
	-- Ignore pet battle and dungeon quests
	if (worldQuestType == LE_QUEST_TAG_TYPE_PET_BATTLE or worldQuestType == LE_QUEST_TAG_TYPE_DUNGEON) then
		WorldQuestGroupFinder.prefixedPrint(L["This type of world quest is not allowed."])
		return false
	end
	-- Finding current zone activity ID
	for k,v in pairs( activityIDs ) do
		if (C_LFGList.GetActivityInfoExpensive(k)) then
			currentZone = k
			foundZone = true
		end
	end
	if (foundZone) then
		C_LFGList.RemoveListing()
		-- Clear existing applications
		if (C_LFGList.GetNumApplications() > 0) then
			for k,v in pairs( C_LFGList.GetApplications() ) do
				C_LFGList.CancelApplication(v)
			end
		end
		WorldQuestGroupFinder.prefixedPrint(string.format(L["Searching for a group for world quest |c00bfffff%s|c00ffffff..."], title))
		-- Search for existing groups
		C_LFGList.ClearSearchResults()
		C_LFGList.Search(1, "#WQ:"..wqID.."#", 0, 4, C_LFGList.GetLanguageSearchFilter());
		-- Delay to let the search finish
		C_Timer.After(2, function()
			local applicationsCount = 0
			local blacklistedApplicationsCount = 0
			local searchCount, searchResults = C_LFGList.GetSearchResults()
			local currentPlayers = 1;
			if (IsInGroup()) then
				currentPlayers = GetNumGroupMembers()
			end
			for k, v in pairs( searchResults ) do
				local _,_,name,description,_,ilvl,_,_,_,_,_,_,author,members = C_LFGList.GetSearchResultInfo(v)
				-- Double check for description content, required ilvl and group members
				if (string.find(description, "#WQ:"..wqID.."#") ~= nil and GetAverageItemLevel() > ilvl and members + currentPlayers <= 5 and author ~= UnitName("player")) then
					if (blacklistedLeaders[author] == true) then
						blacklistedApplicationsCount = blacklistedApplicationsCount + 1
					else 
						foundGroup = true;
						local canBeTank = LFDQueueFrameRoleButtonTank.checkButton:GetChecked()
						local canBeHealer = LFDQueueFrameRoleButtonHealer.checkButton:GetChecked()
						local canBeDamager = LFDQueueFrameRoleButtonDPS.checkButton:GetChecked()
						if ((canBeTank or canBeHealer or canBeDamager) == false) then
							canBeTank, canBeHealer, canBeDamager = UnitGetAvailableRoles("player")
						end
						C_LFGList.ApplyToGroup(v, "WorldQuestGroupFinder User", canBeTank, canBeHealer, canBeDamager)
						applicationsCount = applicationsCount + 1
					end
				end
			end
			C_LFGList.ClearSearchResults()
			-- No group found, creating a new one
			if (foundGroup == false) then
				if (blacklistedApplicationsCount > 0) then
					WorldQuestGroupFinder.prefixedPrint(string.format(L["You have not been applied to %d group(s) because their leader was blacklisted. You can use |c00bfffff/wqgf unbl |c00ffffffto clear the blacklist."], blacklistedApplicationsCount))
				end
				local shortTitle = strsub(title, 0, 31)
				local autoInviteAllState = WorldQuestGroupFinderConf.GetConfigValue("autoinvite") and WorldQuestGroupFinderConf.GetConfigValue("autoinviteUsers")
				C_LFGList.CreateListing(currentZone, shortTitle, 0, 0, "", "Doing the world quest \""..title.."\" in "..activityIDs[currentZone]..". Automatically created by World Quest Group Finder. #WQ:"..wqID.."#", autoInviteAllState)
				WorldQuestGroupFinder.prefixedPrint(string.format(L["A new group finder entry has been created for |c00bfffff%s|c00ffffff in |c00bfffff%s|c00ffffff."], title, activityIDs[currentZone]))
				WorldQuestGroupFinder.HandleWorldQuestStart(wqID)
			else 
				WorldQuestGroupFinder.prefixedPrint(string.format(L["You have been applied to |c00bfffff%d|c00ffffff group(s) for the world quest |c00bfffff%s|c00ffffff."], applicationsCount, title))
			end
			return true
		end)
	else
		WorldQuestGroupFinder.prefixedPrint(L["You are not in the right location for this world quest."])
		return false
	end
end

function WorldQuestGroupFinder.HandleWorldQuestStart(wqID)
	WorldQuestGroupFinder.AddWorldQuestToTracker(wqID)	
	if (WorldQuestGroupFinder.AttachBorderToCurrentWQ(wqID)) then
		pendingApplications = {}
		blacklistedLeaders = {}
		currentWQ = wqID
		WorldQuestGroupFinderConf.SetConfigValue("savedCurrentWQ", currentWQ)
		if (IsInGroup() and UnitIsGroupLeader("player")) then
			SendAddonMessage(BROADCAST_PREFIX, "#WQS:"..wqID.."#", "PARTY")
		end
		tempWQ = nil
		return true
	else
		return false
	end
end
	
function WorldQuestGroupFinder.HandleWorldQuestEnd(wqID)
	currentWQFrame:Hide()
	WorldQuestGroupFinderConf.SetConfigValue("savedCurrentWQ", nil)
	BonusObjectiveTracker_UntrackWorldQuest(wqID)
	if (IsInGroup() and UnitIsGroupLeader("player")) then
		SendAddonMessage(BROADCAST_PREFIX, "#WQE:"..wqID.."#", "PARTY")
	end
	currentWQ = nil
	tempWQ = nil
end
	
function WorldQuestGroupFinder.AttachBorderToCurrentWQ(wqID)
	local targetBlock = WorldQuestGroupFinder.FindWQBlock(wqID)
	if (targetBlock) then
	
		local xOffset = -12
		local yOffset = 10
		
		local trackerWidth, trackerHeight = ObjectiveTrackerFrame:GetSize()
		local blockWidth, blockHeight = targetBlock:GetSize()
			
		currentWQFrame:SetClampedToScreen(true)
		currentWQFrame:SetFrameStrata("MEDIUM")
		currentWQFrame:SetToplevel(false) 
		currentWQFrame:RegisterForDrag("LeftButton")
		
		currentWQFrame:SetSize(blockWidth+18,blockHeight+12)
		currentWQFrame:SetMovable(false)
		currentWQFrame:EnableMouse(false)
		
		currentWQFrame:SetPoint("TOPLEFT", targetBlock,"TOPLEFT", xOffset, yOffset)
		currentWQFrame:SetParent(targetBlock)
			
		currentWQFrame:SetFrameStrata("LOW")
		currentWQFrame:SetFrameLevel(0)

		currentWQFrame:SetBackdrop({
		  bgFile="Interface\\QuestionFrame\\question-background.blp", 
		  edgeFile="Interface\\DialogFrame\\UI-DialogBox-Gold-Border", 
		  tile=false, edgeSize=16, --tileSize=16,
		  insets={left=5, right=5, top=5, bottom=5}
		})
		  
		currentWQFrame:Show()
		
		return currentWQFrame
	else 
		return false
	end
end

StaticPopupDialogs["WORLD_QUEST_FINISHED_LEADER_PROMPT"] = {
	text = L["You have completed the world quest. Would you like to leave the group or unlist it from the Group Finder?"],
	button1 = L["Leave"],
	button2 = L["Stay"],
	button3 = L["Unlist"],
	OnAccept = function()
		LeaveParty()
	end,
	OnAlt = function()
		C_LFGList.RemoveListing()
	end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	preferredIndex = 3,  
}

StaticPopupDialogs["WORLD_QUEST_FINISHED_PROMPT"] = {
	text = L["You have completed the world quest. Would you like to leave the group?"],
	button1 = "Leave",
	button2 = "Stay",  
	OnAccept = function()
		LeaveParty()
	end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	preferredIndex = 3,  
}

StaticPopupDialogs["NEW_WORLD_QUEST_PROMPT"] = {
	text = L["You are currently grouped for another world quest.\nAre you sure to want to start another one?"],
	button1 = "Yes",
	button2 = "Cancel",  
	OnAccept = function()
		LeaveParty()
		C_Timer.After(1, function()
			WorldQuestGroupFinder.InitWQGroup(tempWQ)
		end)
		tempWQ = nil
	end,
	OnCancel = function() 
		tempWQ = nil
	end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	enterClicksFirstButton = true,
	preferredIndex = 3,  
}

StaticPopupDialogs["WORLD_QUEST_ENTERED_PROMPT"] = {
	text = L["You have entered a new world quest area.\nWould you like to find a group for \"%s\"?"],
	button1 = "Yes",
	button2 = "No",  
	OnAccept = function()
		WorldQuestGroupFinder.InitWQGroup(tempWQ)
		tempWQ = nil
	end,
	OnCancel = function() 
		tempWQ = nil
	end,
	timeout = 20,
	whileDead = false,
	hideOnEscape = true,
	preferredIndex = 3,  
}

StaticPopupDialogs["WORLD_QUEST_ENTERED_SWITCH_PROMPT"] = {
	text = L["You have entered a new world quest area, but are currently grouped for another world quest.\nWould you like to leave your current group and find another for \"%s\"?"],
	button1 = "Yes",
	button2 = "No",  
	OnAccept = function()
		LeaveParty()
		C_Timer.After(1, function()
			WorldQuestGroupFinder.InitWQGroup(tempWQ)
			tempWQ = nil
		end)
	end,
	OnCancel = function() 
		tempWQ = nil
	end,
	timeout = 20,
	whileDead = false,
	hideOnEscape = true,
	preferredIndex = 3,  
}

function WorldQuestGroupFinder.SendWQCompletionPartyNotification (questID)
	local title, factionID, tagID, tagName, worldQuestType, rarity, elite, tradeskillLineIndex = WorldQuestGroupFinder.GetQuestInfo(questID)
	SendChatMessage("[WQGF] World quest \""..title.."\" completed! "..L[":)"], "PARTY", "", "");
end

function WorldQuestGroupFinder.ShowLeavePrompt()
	if (UnitIsGroupLeader("player")) then
		StaticPopup_Show ("WORLD_QUEST_FINISHED_LEADER_PROMPT")
	else 
		StaticPopup_Show ("WORLD_QUEST_FINISHED_PROMPT")
	end
end

function WorldQuestGroupFinder.handleCMD(msg, editbox)
	if (string.lower(msg) == "config") then
		InterfaceOptionsFrame_OpenToCategory("WorldQuestGroupFinder")
	elseif (string.lower(msg) == "debug") then
		if (currentWQ ~= nil) then
			WorldQuestGroupFinder.prefixedPrint(string.format(L["Current world quest ID is |c00bfffff%s|c00ffffff."], currentWQ))
		else 
			WorldQuestGroupFinder.prefixedPrint(L["No current world quest."])
		end
		print(L["World quest zones entered this session :"])
		WorldQuestGroupFinder.dumpTable(seenWorldQuests)
	elseif (string.lower(msg) == "showconfig") then
		WorldQuestGroupFinder.dumpTable(WorldQuestGroupFinderConfig)
	elseif (string.lower(msg) == "unbl") then
		blacklistedLeaders = {}
		WorldQuestGroupFinder.prefixedPrint(L["The leaders blacklist has been cleared."])
	else
		WorldQuestGroupFinder.help()
	end
end

function WorldQuestGroupFinder.help()
	print(L["|c00bfffffSlash commands (/wqgf):"])
	print(L["|c00bfffff /wqgf config : Open addon configuration"])
	print(L["|c00bfffff /wqgf unbl : Clear the leaders blacklist"])
end

function WorldQuestGroupFinder.prefixedPrint(text)
	print("|c00bfffff[WQGF]|c00ffffff "..text)
end 

function WorldQuestGroupFinder.dumpTable(t)
	if type(t) == "table" then
		for k, v in pairs( t ) do
			print(k, v)
		end
	else 
		print(t)
	end
end

function WorldQuestGroupFinder.FindWQBlock(wqID)
	local tracker = ObjectiveTrackerFrame
	-- CAST OR DIE!!!
	wqID = tonumber(wqID)
	
	if (not tracker.initialized) then
		return
	end
	for i = 1, #tracker.MODULES do
		local module = tracker.MODULES [i]
		for blockName, usedBlock in pairs (module.usedBlocks) do
			if (usedBlock.id) then
				if (wqID == usedBlock.id) then
					return usedBlock
				end
			end
		end
	end
	return false
end